It is the implementation process description for  measuring spatial accessibility to facilities based on multiple transportation modes considered residential transportation modes choices (MTM-RTMC).

#### 1. Prerequisites for operation

​    (1) We should sign up for Amap Maps account and get the key from Amap Developer Center through URL https://lbs.amap.com/
​    (2) The required basic software environments are: Python 3.6, ArcGIS 10.2, Oracle 10.1, MongoDB 3.1.
​    (3) In this experiment, the MongoDB collection data, "rtmcp_mtm_gm.json" data, should be restored in the MongoDB database. The import command is as "mongoimport --db test --collection rtmcp_mtm_gm --file D:\ rtmcp_mtm_gm.json". The basic data structure of demand object stored in "rtmcp_mtm_gm" collection is as following in **Appendix A**.
​    (4) In this experiment, the spatial statistical units, "street_block.shp" data, should be restored into the Oracle database through ArcGIS SDE. Creating a geodatabase in Oracle can refer to URL "<https://resources.arcgis.com/en/help/main/10.2/index.html#//002n000000tn000000>". The "street_block.shp" data is replaced by a mock data of my spatial statistical units experimental data, called "street_block_mock_data.shp" data. The "street_block_mock_data.shp" data inherit all fields from  my spatial statistical units experimental data.
​    **Why we use mock data to express our spatial statistical units data?**
​    Our spatial statistical units data, street block data in Nanjing, involve with mapping geographic information security. The street block data is similar to traffic analysis area (TAZ) data, very detailed. 
​    **The generation process for "street_block_mock_data.shp"  data in ArcMap is as following:**
​        [data]street block polygon data  --->[arcmap tool operating]feature to point ---> [data]street block central points data ---->[arcmap tool operating]buffer ---> [data] cricle polygons data --->[arcmap tool operating]feature envelope to polygon ---> [data] street block mock data.
​    **The fields description of the "street_block_mock_data.shp"  data is as following:**
​        <u>OBJECTID_12</u> - the key field of the street_block_mock_data.shp data;
​        <u>OBJECTID</u> - the key field of the street_block.shp data.
​        <u>SUM_ACTURA</u> - the population count of the street block.
​        <u>X</u> - the location x of the street block's central point.
​        <u>Y</u> - the location y of the street block's central point.
​        <u>SINGLE</u> - the accessibility value calculated by the STM G2SFCA model.
​        <u>MTM</u> - the accessibility value calculated by the MTM G2SFCA model.
​        <u>MTM_RTMCP</u> - the accessibility value calculated by the MTM_RTMCP G2SFCA model.

#### 2. The code implementation

**Step 1:** We initialize the "rtmcp_mtm_gm" collection through _01_initial_mogodb_rtmcp_mtm_gm_collection.py.
**Step 2:** A method for real-time navigation time and distance between demand points and provider points is developed based on the Amap Maps route planning API.
        Four route planning data of MTM from internet maps include driving, walking, public transportation, and bicycling.
        	    _02_route_plan_calculator.py
                    _02_route_planner.py
                    _02_route_planning_queue.py
                    _02_save_route_planning_data_for_queue.py
**Step 3:** We design three accessibility models, including the STM G2SFCA, the MTM G2SFCA, and the MTM-RTMC G2SFCA, and calculate the accessibility results of the three models.
                    _03_calculate_softmax_weights_for_RTMCP_MTM.py
                    _03_STM_G2SFCA_calculation.py
                    _03_MTM_G2SFCA_calculation.py
                    _03_MTM_RTMC_G2SFCA_calculation.py
**Step 4:** To explain the experimental results intuitively, we need to express the results spatially by those coding files,including:
                    _04_extract_accessibility_valut_to_ArcSDE_shp.py
                    _04_extract_NCHGZ_distance_time_to_shp.py
            The mapping symbolization process was completed by ArcMap10.2.

In addition, we developed a series of basic functions to support the implementation, such as: the function to operate MongoDB, mongo_operater.py

#### 3. Appendix A

##########################################################Appendix A ############## one demand object data structure of MongoDB collection data ##############################################
{
        "_id" : ObjectId("5db7fa6e8abead46fc8074e5"),
        "demand_id" : 34,
        "x" : 118.88721444,
        "y" : 31.2587319,
        "pd" : 0.086,   
        "pdd" : 0.03318611,
        "provider" : [
                {
                        "provider_id" : 1661,
                        "x" : 118.77423521,
                        "y" : 32.05267463,
                        "dc" : 157,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.23,
                                "TP_D" : 105.69,
                                "D_T" : 1.66,
                                "D_D" : 110.34,
                                "W_T" : 21.52,
                                "W_D" : 96.86,
                                "B_T" : 6.52,
                                "B_D" : 97.75
                        },
                        "softmax" : {
                                "TP" : 0.243,
                                "D" : 0.602,
                                "W" : 0.039,
                                "B" : 0.115
                        }
                },
                {
                        "provider_id" : 1662,
                        "x" : 118.70260302,
                        "y" : 31.98399565,
                        "dc" : 235,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.05,
                                "TP_D" : 102.57,
                                "D_T" : 1.51,
                                "D_D" : 106.25,
                                "W_T" : 21.19,
                                "W_D" : 95.38,
                                "B_T" : 6.4,
                                "B_D" : 95.96
                        },
                        "softmax" : {
                                "TP" : 0.237,
                                "D" : 0.613,
                                "W" : 0.04,
                                "B" : 0.11
                        }
                },
                {
                        "provider_id" : 1665,
                        "x" : 118.76,
                        "y" : 32.05,
                        "dc" : 26,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.28,
                                "TP_D" : 106.23,
                                "D_T" : 1.63,
                                "D_D" : 110.07,
                                "W_T" : 21.49,
                                "W_D" : 96.71,
                                "B_T" : 6.54,
                                "B_D" : 98.04
                        },
                        "softmax" : {
                                "TP" : 0.233,
                                "D" : 0.614,
                                "W" : 0.04,
                                "B" : 0.114
                        }
                },
                {
                        "provider_id" : 1671,
                        "x" : 118.772,
                        "y" : 32.702,
                        "dc" : 13,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 888,
                                "TP_D" : 888,
                                "D_T" : 2.7,
                                "D_D" : 202.05,
                                "W_T" : 888,
                                "W_D" : 888,
                                "B_T" : 11.97,
                                "B_D" : 179.55
                        },
                        "softmax" : {
                                "TP" : 0.06,
                                "D" : 0.656,
                                "W" : 0.06,
                                "B" : 0.224
                        }
                },
                {
                        "provider_id" : 1684,
                        "x" : 118.933,
                        "y" : 32.095,
                        "dc" : 6,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.52,
                                "TP_D" : 118.92,
                                "D_T" : 1.73,
                                "D_D" : 125.37,
                                "W_T" : 22.85,
                                "W_D" : 102.82,
                                "B_T" : 7.01,
                                "B_D" : 105.2
                        },
                        "softmax" : {
                                "TP" : 0.23,
                                "D" : 0.613,
                                "W" : 0.039,
                                "B" : 0.117
                        }
                },
                {
                        "provider_id" : 1672,
                        "x" : 118.739,
                        "y" : 32.081,
                        "dc" : 26,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.85,
                                "TP_D" : 110.94,
                                "D_T" : 1.67,
                                "D_D" : 114.29,
                                "W_T" : 22.52,
                                "W_D" : 101.33,
                                "B_T" : 6.89,
                                "B_D" : 103.29
                        },
                        "softmax" : {
                                "TP" : 0.197,
                                "D" : 0.649,
                                "W" : 0.041,
                                "B" : 0.114
                        }
                },
                {
                        "provider_id" : 1666,
                        "x" : 118.736,
                        "y" : 32.06,
                        "dc" : 23,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.28,
                                "TP_D" : 110.06,
                                "D_T" : 1.59,
                                "D_D" : 112.04,
                                "W_T" : 22.16,
                                "W_D" : 99.71,
                                "B_T" : 6.76,
                                "B_D" : 101.47
                        },
                        "softmax" : {
                                "TP" : 0.226,
                                "D" : 0.624,
                                "W" : 0.041,
                                "B" : 0.109
                        }
                },
                {
                        "provider_id" : 1670,
                        "x" : 118.771,
                        "y" : 32.041,
                        "dc" : 14,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.15,
                                "TP_D" : 103.96,
                                "D_T" : 1.53,
                                "D_D" : 108.48,
                                "W_T" : 21.28,
                                "W_D" : 95.76,
                                "B_T" : 6.47,
                                "B_D" : 97.09
                        },
                        "softmax" : {
                                "TP" : 0.232,
                                "D" : 0.617,
                                "W" : 0.04,
                                "B" : 0.11
                        }
                },
                {
                        "provider_id" : 1682,
                        "x" : 118.752,
                        "y" : 32.237,
                        "dc" : 3,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.93,
                                "TP_D" : 131.13,
                                "D_T" : 1.92,
                                "D_D" : 140.09,
                                "W_T" : 888,
                                "W_D" : 888,
                                "B_T" : 8.19,
                                "B_D" : 122.78
                        },
                        "softmax" : {
                                "TP" : 0.277,
                                "D" : 0.524,
                                "W" : 0.035,
                                "B" : 0.163
                        }
                },
                {
                        "provider_id" : 1687,
                        "x" : 118.844,
                        "y" : 31.95,
                        "dc" : 8,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.36,
                                "TP_D" : 93.86,
                                "D_T" : 1.57,
                                "D_D" : 104.39,
                                "W_T" : 18.73,
                                "W_D" : 84.27,
                                "B_T" : 5.68,
                                "B_D" : 85.19
                        },
                        "softmax" : {
                                "TP" : 0.22,
                                "D" : 0.61,
                                "W" : 0.038,
                                "B" : 0.131
                        }
                },
                {
                        "provider_id" : 1689,
                        "x" : 118.865,
                        "y" : 31.322,
                        "dc" : 20,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 0.99,
                                "TP_D" : 8.95,
                                "D_T" : 0.26,
                                "D_D" : 8.71,
                                "W_T" : 1.82,
                                "W_D" : 8.17,
                                "B_T" : 0.56,
                                "B_D" : 8.33
                        },
                        "softmax" : {
                                "TP" : 0.076,
                                "D" : 0.644,
                                "W" : 0.049,
                                "B" : 0.231
                        }
                },
                {
                        "provider_id" : 1690,
                        "x" : 118.882,
                        "y" : 32.122,
                        "dc" : 2,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.75,
                                "TP_D" : 116.44,
                                "D_T" : 1.73,
                                "D_D" : 121.45,
                                "W_T" : 23.72,
                                "W_D" : 106.74,
                                "B_T" : 7.14,
                                "B_D" : 107.16
                        },
                        "softmax" : {
                                "TP" : 0.215,
                                "D" : 0.63,
                                "W" : 0.04,
                                "B" : 0.115
                        }
                },
                {
                        "provider_id" : 1691,
                        "x" : 118.787,
                        "y" : 32.022,
                        "dc" : 8,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 2.97,
                                "TP_D" : 99.71,
                                "D_T" : 1.55,
                                "D_D" : 105.18,
                                "W_T" : 20.59,
                                "W_D" : 92.65,
                                "B_T" : 6.22,
                                "B_D" : 93.28
                        },
                        "softmax" : {
                                "TP" : 0.253,
                                "D" : 0.594,
                                "W" : 0.04,
                                "B" : 0.113
                        }
                },
                {
                        "provider_id" : 1692,
                        "x" : 118.72,
                        "y" : 31.981,
                        "dc" : 6,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3,
                                "TP_D" : 102.69,
                                "D_T" : 1.45,
                                "D_D" : 104.74,
                                "W_T" : 21.07,
                                "W_D" : 94.82,
                                "B_T" : 6.36,
                                "B_D" : 95.37
                        },
                        "softmax" : {
                                "TP" : 0.228,
                                "D" : 0.623,
                                "W" : 0.041,
                                "B" : 0.108
                        }
                },
                {
                        "provider_id" : 1693,
                        "x" : 118.853,
                        "y" : 32.035,
                        "dc" : 9,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.21,
                                "TP_D" : 106.25,
                                "D_T" : 1.66,
                                "D_D" : 110.33,
                                "W_T" : 21.18,
                                "W_D" : 95.29,
                                "B_T" : 6.39,
                                "B_D" : 95.78
                        },
                        "softmax" : {
                                "TP" : 0.241,
                                "D" : 0.601,
                                "W" : 0.039,
                                "B" : 0.119
                        }
                },
                {
                        "provider_id" : 1694,
                        "x" : 118.804,
                        "y" : 32.099,
                        "dc" : 6,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.25,
                                "TP_D" : 110.83,
                                "D_T" : 1.72,
                                "D_D" : 114.12,
                                "W_T" : 22.59,
                                "W_D" : 101.64,
                                "B_T" : 6.85,
                                "B_D" : 102.69
                        },
                        "softmax" : {
                                "TP" : 0.25,
                                "D" : 0.597,
                                "W" : 0.04,
                                "B" : 0.113
                        }
                },
                {
                        "provider_id" : 1695,
                        "x" : 119.027,
                        "y" : 31.632,
                        "dc" : 6,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.06,
                                "TP_D" : 62.34,
                                "D_T" : 0.87,
                                "D_D" : 53.86,
                                "W_T" : 11.31,
                                "W_D" : 50.89,
                                "B_T" : 3.45,
                                "B_D" : 51.79
                        },
                        "softmax" : {
                                "TP" : 0.106,
                                "D" : 0.722,
                                "W" : 0.046,
                                "B" : 0.126
                        }
                },
                {
                        "provider_id" : 1696,
                        "x" : 118.84,
                        "y" : 32.342,
                        "dc" : 9,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 4.12,
                                "TP_D" : 145.56,
                                "D_T" : 2.16,
                                "D_D" : 152.31,
                                "W_T" : 888,
                                "W_D" : 888,
                                "B_T" : 9.22,
                                "B_D" : 138.38
                        },
                        "softmax" : {
                                "TP" : 0.29,
                                "D" : 0.518,
                                "W" : 0.035,
                                "B" : 0.157
                        }
                },
                {
                        "provider_id" : 1697,
                        "x" : 118.714,
                        "y" : 32.106,
                        "dc" : 4,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 4.13,
                                "TP_D" : 124.04,
                                "D_T" : 1.87,
                                "D_D" : 125.08,
                                "W_T" : 23.46,
                                "W_D" : 105.56,
                                "B_T" : 7.47,
                                "B_D" : 112.12
                        },
                        "softmax" : {
                                "TP" : 0.197,
                                "D" : 0.65,
                                "W" : 0.041,
                                "B" : 0.113
                        }
                },
                {
                        "provider_id" : 1698,
                        "x" : 118.736,
                        "y" : 32.067,
                        "dc" : 2,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.75,
                                "TP_D" : 112.37,
                                "D_T" : 1.6,
                                "D_D" : 112.82,
                                "W_T" : 22.27,
                                "W_D" : 100.19,
                                "B_T" : 6.82,
                                "B_D" : 102.25
                        },
                        "softmax" : {
                                "TP" : 0.19,
                                "D" : 0.657,
                                "W" : 0.041,
                                "B" : 0.112
                        }
                },
                {
                        "provider_id" : 1701,
                        "x" : 118.744,
                        "y" : 32.093,
                        "dc" : 1,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.71,
                                "TP_D" : 112.75,
                                "D_T" : 1.77,
                                "D_D" : 116.35,
                                "W_T" : 22.74,
                                "W_D" : 102.34,
                                "B_T" : 6.92,
                                "B_D" : 103.73
                        },
                        "softmax" : {
                                "TP" : 0.218,
                                "D" : 0.626,
                                "W" : 0.04,
                                "B" : 0.117
                        }
                },
                {
                        "provider_id" : 1702,
                        "x" : 118.632,
                        "y" : 32.05,
                        "dc" : 3,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.64,
                                "TP_D" : 118.07,
                                "D_T" : 1.77,
                                "D_D" : 122.22,
                                "W_T" : 24.72,
                                "W_D" : 111.25,
                                "B_T" : 8.14,
                                "B_D" : 122.03
                        },
                        "softmax" : {
                                "TP" : 0.239,
                                "D" : 0.623,
                                "W" : 0.043,
                                "B" : 0.095
                        }
                },
                {
                        "provider_id" : 1703,
                        "x" : 118.787,
                        "y" : 32.028,
                        "dc" : 1,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.03,
                                "TP_D" : 103.24,
                                "D_T" : 1.57,
                                "D_D" : 106.38,
                                "W_T" : 20.75,
                                "W_D" : 93.39,
                                "B_T" : 6.28,
                                "B_D" : 94.22
                        },
                        "softmax" : {
                                "TP" : 0.243,
                                "D" : 0.603,
                                "W" : 0.04,
                                "B" : 0.114
                        }
                },
                {
                        "provider_id" : 1704,
                        "x" : 118.789,
                        "y" : 32.056,
                        "dc" : 1,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.22,
                                "TP_D" : 103.87,
                                "D_T" : 1.61,
                                "D_D" : 109.95,
                                "W_T" : 21.47,
                                "W_D" : 96.63,
                                "B_T" : 6.5,
                                "B_D" : 97.45
                        },
                        "softmax" : {
                                "TP" : 0.241,
                                "D" : 0.605,
                                "W" : 0.04,
                                "B" : 0.114
                        }
                },
                {
                        "provider_id" : 1664,
                        "x" : 118.888,
                        "y" : 31.933,
                        "dc" : 8,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3.08,
                                "TP_D" : 105.22,
                                "D_T" : 1.42,
                                "D_D" : 101.71,
                                "W_T" : 18.16,
                                "W_D" : 81.72,
                                "B_T" : 5.48,
                                "B_D" : 82.2
                        },
                        "softmax" : {
                                "TP" : 0.184,
                                "D" : 0.647,
                                "W" : 0.039,
                                "B" : 0.13
                        }
                },
                {
                        "provider_id" : 1663,
                        "x" : 118.788,
                        "y" : 32.036,
                        "dc" : 6,
                        "name" : "",
                        "travel" : {
                                "TP_T" : 3,
                                "TP_D" : 100.7,
                                "D_T" : 1.56,
                                "D_D" : 106.31,
                                "W_T" : 20.96,
                                "W_D" : 94.34,
                                "B_T" : 6.34,
                                "B_D" : 95.11
                        },
                        "softmax" : {
                                "TP" : 0.254,
                                "D" : 0.595,
                                "W" : 0.04,
                                "B" : 0.112
                        }
                }
        ],
        "gravity_value" : 0.033581801705606894,
        "mtm_gravity_value" : 0.0320566782699743,
        "mtm_rtmcp_gravity_value" : 0.030646793928404747
}

##########################################################Appendix A ##########one demand object data structure of MongoDB collection data##############################################