# -*- coding:utf-8 -*-

"""
purpose:
A method for real-time route planning time and distance between demand points and provider points is developed based on the Amap Maps route planning API.
The demand points are residential communities. The provider points are hospitals in Nanjing.
The retrieved results are stored in MongoDB's providers object
author:*****
datetime:04,21,2019
help:python $this_file.py --help
"""
import  json,  sys, time, traceback
from _02_route_planner import *
from _02_route_planning_queue import RPQueue
from _02_save_route_planning_data_for_queue import SaverDataOfRPQ
import mongo_operater

"""
mongoDB database and collection setting
"""
DB_NAME="test"
COLLECTION_NAME="rtmcp_mtm_gm"

"""
the enter of excuting spider the Amap Maps route planning API
"""
def do_it(from_row,n_rows):
    mongo_operater_obj = mongo_operater.MongoOperater(DB_NAME, COLLECTION_NAME)
    n_fetched = 0
    #Building the request body for each record
    with SaverDataOfRPQ(mongo_operater_obj) as save_data_of_rpq, RPQueue() as route_planner_queue:
        route_planner_queue.bound_save_data_of_rpq(save_data_of_rpq)
        #Getting all the records
        rows_json_list = mongo_operater_obj.find_records(from_row,n_rows)
        n_fetched = len(rows_json_list)
        print('{}\tfetched {}'.format(time.asctime(time.localtime(time.time())), n_fetched))
        for row_json in rows_json_list:
            providers_json_list=row_json["provider"]
            for item_provider in providers_json_list:
                coordinates = [row_json["x"],row_json["y"],item_provider["x"],item_provider["y"]]
                demand_id = row_json["demand_id"]
                for cls in BaseRoutePlanner.CONCRETE_TYPES:
                    # We place each record request body into a request queue object.
                    route_planner_queue += cls(demand_id,item_provider, *coordinates)
    return n_fetched


if __name__ == "__main__":
    n_rows_fatch = do_it(0, 2265)
    print('have affected {0} at {1}'.format( n_rows_fatch,time.asctime(time.localtime(time.time()))))