# -*- coding:utf-8 -*-
"""
Base class: path planning basic object role.
Use：stores the URL format to control whether the return result is parsed in bulk request mode.
"""

class BaseRoutePlanner(object):
    URL = ''
    def __init__(self, demand_id,provider, *args):
        self.API_KEY = " the key from Amap Developer Center"
        cls = type(self)
        self.demand_id = demand_id
        self.provider=provider
        self.batchable = True
        self.uri = '{}?key={}&origin={},{}&destination={},{}'.format(cls.URL, self.API_KEY, *args)
    """
    Parse the parsing logic that returns the result
    """
    def parse_response_body_get_distance_and_time(self, response_body):

        if 'route' in response_body.keys():
            paths = response_body['route']['paths']
            if paths is None or len(paths) == 0:
                return dict(distance=888, time=888)
            else:
                path_0 = paths[0]
                return dict(distance=round(float(path_0['distance']) / 1000, 2),
                            time=round(float(path_0['duration']) / 3600, 2))
        else:
            return dict(distance=888, time=888)



"""
Standard class: walkpath planning object what it does；
Use: stores the walkurl format in bulk request form and parses the return result logic to be consistent with the base class
"""
class Walkinger(BaseRoutePlanner):
    URL = '/v3/direction/walking'

"""
Class: bus path planning object role;
Use: store bus URL format is batch request way parsing return result logic with special
"""
class Buser(BaseRoutePlanner):
    URL = '/v3/direction/transit/integrated'
    CITY_CODE = '025'

    def __init__(self, demand_id,provider, *args):
        super().__init__(demand_id,provider,  *args)
        self.uri += '&city=' + type(self).CITY_CODE

    """
      Parse the parsing logic that returns the result
    """
    def parse_response_body_get_distance_and_time(self, response_body):
        if response_body.__contains__("route") :
            if response_body['route'].__contains__("transits"):
                paths = response_body['route']['transits']
                if paths is None or len(paths) == 0:
                    return dict(time=100)
                else:
                    path_0 = paths[0]
                    # return dict(distance=path_0['distance'], time=path_0['duration'])
                    return dict(distance=round(float(path_0['distance']) / 1000, 2),
                                time=round(float(path_0['duration']) / 3600, 2))
            else:
                return dict(distance=888, time=888)
        else:
            return dict(distance=888, time=888)

"""
Standard class: driving path planning object role；
Use: store driving URL format is the way of bulk request parsing returned results logic consistent with the base class
"""

class Drivinger(BaseRoutePlanner):
    URL = '/v3/direction/driving'

"""
Class: ride path planning object role；
Use: store the URL format of the ride in a non-bulk request way parsing return result logic special
"""

class Bicyclinger(BaseRoutePlanner):
    URL = '/v4/direction/bicycling'

    def __init__(self, object_id, provider, *args):
        super().__init__(object_id, provider, *args)
        self.batchable = False

    """
      Parse the parsing logic that returns the result
    """
    def parse_response_body_get_distance_and_time(self, response_body):
        paths = response_body['data']['paths']
        if paths is None or len(paths) == 0:
            return None
        else:
            path_0 = paths[0]
            # return dict(distance=path_0['distance'], time=path_0['duration'])
            return dict(distance=round(float(path_0['distance']) / 1000, 2),
                        time=round(float(path_0['duration']) / 3600, 2))


"""
Add class attributes
"""
BaseRoutePlanner.CONCRETE_TYPES = (Walkinger, Buser, Drivinger, Bicyclinger)


