# -*- coding:utf-8 -*-
import requests,json,traceback,time
from _02_route_planner import *
API_KEY = 'the key from Amap Developer Center'


"""
Class：The capabilities of such implementations are bulk request, non-bulk request encapsulation, and throughput control capabilities.
"""
class RPQueue(object):
    BATCH_SIZE = 20
    API_SITE = 'http://restapi.amap.com'
    BATCH_URL = '/v3/batch'
    BATCH_URI = '{}{}?key={}'.format(API_SITE, BATCH_URL, API_KEY)
    BATCH_HDR = {'Content-Type': 'application/json;charset=utf-8'}
    REQ_INTERVAL = 0.01  # throughput < 200Hz

    def __init__(self):
        self.request_cache = []
        self.on_each_ok_callback_save_data = None
        self.last_req_time = 0.0

    def __enter__(self):
        return self

    """
    Function: clearing the queue cache before the end.
    """
    def __exit__(self, exc_type, exc_val, exc_tb):
        self._batch_request_of_rpqueue(self.request_cache)

    """
    Function: Adding queue.
    """
    def __iadd__(self, route_planner_object):
        if not isinstance(route_planner_object, BaseRoutePlanner.CONCRETE_TYPES):
            return self
        if route_planner_object.batchable:
            self.request_cache.append(route_planner_object)
            if len(self.request_cache) == type(self).BATCH_SIZE:
                self._batch_request_of_rpqueue([r for r in self.request_cache])
                self.request_cache.clear()
        else:
            self._standalone_request_of_rpqueue(route_planner_object)
        return self

    """
    Function: The callback function that handles the logic for storing the data after the binding request succeeds.
    """
    def bound_save_data_of_rpq(self, callback):
        self.on_each_ok_callback_save_data = callback

    """
    Function: sending a non-bulk request
    """
    def _standalone_request_of_rpqueue(self, route_planner_object):
        cls = type(self)
        self._throughput_control_of_rpqueue(cls.REQ_INTERVAL)
        try:
            reply = requests.get(cls.API_SITE + route_planner_object.uri, timeout=1)
            if 200 <= reply.status_code < 300:
                self.on_each_ok_callback_save_data(reply.json(), route_planner_object)
        except:
            print(cls.API_SITE + route_planner_object.uri, "   time out")

    """
    Function: sending bulk request
    """
    def _batch_request_of_rpqueue(self, route_planner_objects):
        ops = [dict(url=r.uri) for r in route_planner_objects]
        cls = type(self)
        self._throughput_control_of_rpqueue(cls.REQ_INTERVAL * cls.BATCH_SIZE)
        req=""
        res=""
        try:
            reply = requests.post(cls.BATCH_URI, data=json.dumps(dict(ops=ops)),headers=cls.BATCH_HDR,timeout=3)
            if 200 <= reply.status_code < 300:
                for route_planner_object, route_planner_response in zip(route_planner_objects, reply.json()):
                    if 'status' in route_planner_response:
                        if 200 <= route_planner_response['status'] < 300 and self.on_each_ok_callback_save_data is not None:
                            try:
                                self.on_each_ok_callback_save_data(route_planner_response['body'], route_planner_object)
                            except:
                                traceback.print_exc()
        except:
            print("batch  requests.post fail")
            for route_planner_object in route_planner_objects:
                self.on_each_ok_callback_save_data(route_planner_response_body=None,route_planner_object=route_planner_object,batch_fail=True)
    """
    Function: throughput control
    """
    def _throughput_control_of_rpqueue(self, min_interval):
        now_secs = float(time.time())
        remaining_req_interval = self.last_req_time + min_interval - now_secs
        if remaining_req_interval > 0:
            time.sleep(remaining_req_interval)
            self.last_req_time = float(time.time())
        else:
            self.last_req_time = now_secs