# -*- coding:utf-8 -*-
import json,  time, traceback
from _02_route_planner import *

"""
This is a class for saving the Amap API's responds.
"""
class SaverDataOfRPQ(object):
    BUFFER_HIGH_WATER = 200
    def __init__(self, mongo_operater_obj):
        self.result_cache_dict = {}

        self.result_cache_iobjectid_counter_dict={}

        self.mongo_operater_obj = mongo_operater_obj

        self.POI_ATTR_DICT = {
            (Buser.__name__, 'distance'): "TP_D",
            (Buser.__name__, 'time'): "TP_T",
            (Drivinger.__name__, 'distance'): "D_D",
            (Drivinger.__name__, 'time'): "D_T",
            (Bicyclinger.__name__, 'distance'): "B_D",
            (Bicyclinger.__name__, 'time'): "B_T",
            (Walkinger.__name__, 'distance'): "W_D",
            (Walkinger.__name__, 'time'): "W_T"
        }

        self.cursor = None
        self.n_accepted = 0
        self.n_skipped = 0


    def __enter__(self):
        return self

    """
   Function: Clearing the queue cache before the end.
    """
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.mongo_operater_obj.close()


    """
    Function: The request is successfully invoked for data warehousing
    """
    def __call__(self, route_planner_response_body, route_planner_object,batch_fail=False):
        result={}

        if batch_fail:
            result={"distance":888,"time":888}
        else:

            result = route_planner_object.parse_response_body_get_distance_and_time(route_planner_response_body)


        demand_id = route_planner_object.demand_id
        provider = route_planner_object.provider
        provider_id=provider["provider_id"]
        one_intact_rp_id = ("-").join([str(demand_id),str(provider_id)])


        if one_intact_rp_id not in self.result_cache_dict:
            self.result_cache_dict[one_intact_rp_id] = {}

        self.result_cache_dict[one_intact_rp_id][type(route_planner_object).__name__] = result
        poi_attr_dict = self.result_cache_dict[one_intact_rp_id]
        # print("this is a test. The objectid is {0}, the provider_id is {1}, the travel mode is {2}".format(demand_id,provider["provider_id"],type(route_planner_object).__name__))
        if len(poi_attr_dict) == len(BaseRoutePlanner.CONCRETE_TYPES):

            self._update_query_one_provider_per_time(demand_id, provider, poi_attr_dict)
            del self.result_cache_dict[one_intact_rp_id]

    """
   Function: Submit
    """
    def _update_query_one_provider_per_time(self, demand_id,provider, poi_attr):
        travel=provider["travel"]
        # if travel["TP_T"]>=100 :
        travel["TP_T"]=poi_attr["Buser"]["time"]
        travel["TP_D"]=poi_attr["Buser"]["distance"]
        travel["D_T"]=poi_attr["Drivinger"]["time"]
        travel["D_D"]=poi_attr["Drivinger"]["distance"]
        travel["W_T"]=poi_attr["Walkinger"]["time"]
        travel["W_D"]=poi_attr["Walkinger"]["distance"]
        travel["B_T"]=poi_attr["Bicyclinger"]["time"]
        travel["B_D"]=poi_attr["Bicyclinger"]["distance"]
        self.mongo_operater_obj.update_provider_travel_record(travel, demand_id, provider["provider_id"])
        print('{}\tcommitted---demand point is {}, provider point is {}'.format(time.asctime(time.localtime(time.time())),demand_id, provider["provider_id"]))



